function varargout = WSC(varargin)
% This program was developed to perform wavenumber shift calibration in
% FT-IR spectrometry in an interactive mode.
%
% USAGE:
%  [Wavenumbers_Calibrated, Absorbance_Calibrated] = 
%       WSC(Wavenumbers_C, Absorbance_C, Wavenumbers_R, Absorbance_R);
%
% Output arguments:
%  Wavenumbers_Calibrated: as the name shows.
%  Absorbance_Calibrated:  as the name shows.
%
% Input arguments:
%  Wavenumbers_C:          wavenumbers of the spectrum to be calibrated.
%  Absorbance_C:           absorbance of the spectrum to be calibrated.
%  Wavenumbers_R:          wavenumbers of the reference spectrum.
%  Absorbance_R:           absorbance of the reference spectrum.
%
% Attention:
%  - This is not a software package with strong error tolerance.  So no
%    intentional test, esp. do not try to close the program while select 
%    the region (the cursor is the 'full cross hair'). 
%  - There is no error check about the validity of the input arguments,
%    e.g. 'Wavenumbers_C' and 'Absorbance_C' should be of the same length.
%    So be care of the input, otherwise serious problems may occur.
%  - As you may find the function can be called without output arguments.
%      H = WSC returns the handle to a new WSC or the handle to
%      the existing singleton*.
%
% __ 'UserData' domains __
%    * The 'UserData' of the main figure are indices to the translation and
%      the dilation wavenumbers.
%    * The 'UserData' of the axes are original X- and Y- limitations used
%      to restore the whole display of the plots.
%    * The 'UserData' of the slider for the fine adjustment is the number
%      for the flow of the whole calibration process.
%    * The 'UserData' of the 'Proceed' button is the output: calibrated
%      wavenumbers and corresponding absorbance if the whole process is
%      accomplished normally, or two zeros if the process is interrupted
%      before normal accomplishment.
%
% The history of the core codes (without the GUI):
%  - The fifth modification was on Friday of Nov. 4, 2011.
%  - Fourth modification was on Apr. 8, 2005.
%  - Third modification was on Jan. 19, 2005.
%  - Second modification was on Dec. 21, 2004.
%  - First modification was on Dec. 16, 2004.
%  - Created by Limin Shao on Dec. 14, 2004.
%
% The history of the GUI: 
%
%  - Created by Limin Shao on Apr. 12, 2005.

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @WSC_OpeningFcn, ...
                   'gui_OutputFcn',  @WSC_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout == 2
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT


% --- Executes just before WSC is made visible.
function WSC_OpeningFcn(hObject, eventdata, handles, varargin)
% Update handles structure
guidata(hObject, handles);

% <<<< Plot of the curves starts.
Wavenumbers_C = varargin{1}(:);
Absorbance_C = varargin{2}(:);
Wavenumbers_R = varargin{3}(:);
Absorbance_R = varargin{4}(:);
% Make them column vectors.

global ISASCENDINGORDER

ISASCENDINGORDER = 1;
if Wavenumbers_C(1) > Wavenumbers_C(end)
    Wavenumbers_C = Wavenumbers_C(end:-1:1);
    Absorbance_C = Absorbance_C(end:-1:1);
    ISASCENDINGORDER = 0;
end;
if Wavenumbers_R(1) > Wavenumbers_R(end)
    Wavenumbers_R = Wavenumbers_R(end:-1:1);
    Absorbance_R = Absorbance_R(end:-1:1);
end;
% get calibrated and reference spectra, and if necessary resort them in the
% order of ascending wavenumbers then set a flag (a global variable).

Hndl_Plot_C = plot(Wavenumbers_C, Absorbance_C);
set(Hndl_Plot_C, 'Tag', 'Plot_Calibrated');
hold on;
Hndl_Plot_R = plot(Wavenumbers_R, Absorbance_R, 'r');
set(Hndl_Plot_R, 'Tag', 'Plot_Reference');
legend('Calibrated', 'Reference');
xlabel('Wavenumbers'); ylabel('Absorbance');
grid on;
% >>>> Plot of the curves ends.

set(gca, 'UserData', [get(gca, 'XLim'); get(gca, 'YLim')]);
% Save the original ranges of the coordinates of the plot.

set(handles.Sldr_Wavenumber_Adjustment, 'UserData', 1);
% Set number of steps in the wavenumber calibration procedure.

set(handles.Sldr_Wavenumber_Adjustment, 'Enable', 'Off');
set(handles.TglBtn_ZoomIn, 'Enable', 'Off');
set(handles.Btn_Restore, 'Enable', 'Off');
% Disable three UI controls unused now.


% Function used to send back computation results.
function varargout = WSC_OutputFcn(hObject, eventdata, handles) 
uiwait;
FinalResult = get(handles.Btn_Proceed, 'UserData');
if nargout ~= 2
    global WAVENUMBERS_CALIBRATED ABSORBANCE_CALIBRATED;
    WAVENUMBERS_CALIBRATED = FinalResult(:, 1);
    ABSORBANCE_CALIBRATED = FinalResult(:, 2);
    evalin('base', 'global WAVENUMBERS_CALIBRATED ABSORBANCE_CALIBRATED');
    disp(' ');
    disp('You have assigned no output arguments.');
    disp('The results are now store in two global variables:');
    disp(' ''WAVENUMBERS_CALIBRATED'' and ''ABSORBANCE_CALIBRATED''');
    disp('And you can get access to the two variables in the workspace.');
    disp(' ');
else
    varargout{1} = FinalResult(:, 1);
    varargout{2} = FinalResult(:, 2);
end;
delete(hObject);
% ____ End of function 'WSC_OutputFcn'.


% Function used to restore the original coordinate ranges values of which
% are stored in the 'UserData' of current axes.
function Btn_Restore_Callback(hObject, eventdata, handles)
OriginalCoordinateRanges = get(gca, 'UserData');
set(gca, 'XLim', OriginalCoordinateRanges(1, :));
set(gca, 'YLim', OriginalCoordinateRanges(2, :));
% ____ End of function 'Btn_Restore_Callback'.


% Function used to activate zoom-in operation.
function TglBtn_ZoomIn_Callback(hObject, eventdata, handles)
if get(hObject,'Value') == get(hObject, 'Max')
    set(gcf, 'Pointer', 'crosshair');
    set(gca, 'ButtonDownFcn', 'WSC(''ZoomInCommand'',gcbo,[],guidata(gcbo))');
else
    set(gca, 'ButtonDownFcn', ' ');
    set(gcf, 'Pointer', 'arrow');
end;
% ____ End of function 'TglBtn_ZoomIn_Callback'.


% Function used to finely adjust the wavenumber shift of the calibrated
% spectrum.
function Sldr_Wavenumber_Adjustment_Callback(hObject, eventdata, handles)
FlowNumber = get(hObject, 'UserData');
% The two steps in this function correspond to the operations #2 and #4
% in function 'Btn_Proceed_Callback'.  Go there for detailed informations.
% The number of the current operation of the whole process.   Note that 
% fine adjustment is always done after the main adjustment (by clicking 
% the 'Proceed' button).  So the 'FlowNumber' here is always greater by 
% 1.  Please do not be perplexed.
Hndl_Plot_C = findobj('Tag', 'Plot_Calibrated');
Wavenumbers_C = get(Hndl_Plot_C, 'xdata');
Absorbance_C = get(Hndl_Plot_C, 'ydata');

switch FlowNumber
    case 3,
        delete(Hndl_Plot_C);
        Hndl_Plot_C = plot(Wavenumbers_C + get(hObject, 'Value'), Absorbance_C);
        set(Hndl_Plot_C, 'Tag', 'Plot_Calibrated');
        set(hObject, 'Value', 0);        
    case 5,
        temp = get(gcf, 'UserData');
        Index_Translation = temp(1);
        Index_Dilation = temp(2);
        CalibratedWavenumber_Dilation = Wavenumbers_C(Index_Dilation) + get(hObject, 'Value');
        CalibratedWavenumber_Translation = Wavenumbers_C(Index_Translation);
        CalibratedWavenumberIncrement = (CalibratedWavenumber_Dilation - ...
            CalibratedWavenumber_Translation)/...
            (Index_Dilation - Index_Translation);
        DataLength = length(Wavenumbers_C);
        CalibratedWavenumbers = zeros(DataLength, 1);
        CalibratedWavenumbers(Index_Translation-1:-1:1) = CalibratedWavenumber_Translation - ...
            (1:Index_Translation-1)'*CalibratedWavenumberIncrement;
        CalibratedWavenumbers(Index_Translation) = CalibratedWavenumber_Translation;
        CalibratedWavenumbers(Index_Translation+1:DataLength) = CalibratedWavenumber_Translation + ...
            (1:DataLength - Index_Translation)'*CalibratedWavenumberIncrement;
        delete(Hndl_Plot_C);
        Hndl_Plot_C = plot(CalibratedWavenumbers, Absorbance_C);
        set(Hndl_Plot_C, 'Tag', 'Plot_Calibrated');
        set(hObject, 'Value', 0);
end;
% ____ End of function 'Sldr_Wavenumber_Adjustment_Callback'.


% Function for the slider (UI control).
function Sldr_Wavenumber_Adjustment_CreateFcn(hObject, eventdata, handles)
if isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor',[.9 .9 .9]);
end
% ____ End of function 'Sldr_Wavenumber_Adjustment_CreateFcn'.


% Function used to regulate the flow of the wavenumber shift calibration.
function Btn_Proceed_Callback(hObject, eventdata, handles)
FlowNumber = get(handles.Sldr_Wavenumber_Adjustment, 'UserData');
Hndl_Plot_R = findobj('Tag', 'Plot_Reference');
Wavenumbers_R = get(Hndl_Plot_R, 'XData');
Absorbance_R = get(Hndl_Plot_R, 'YData');
Hndl_Plot_C = findobj('Tag', 'Plot_Calibrated');
Wavenumbers_C = get(Hndl_Plot_C, 'XData');
Absorbance_C = get(Hndl_Plot_C, 'YData');

switch FlowNumber
    case 1, % To start the caliration process.
        set(handles.STxt_Message, 'String', Messages(1));
        UIControls_Enable(handles, 'On');
        set(handles.Sldr_Wavenumber_Adjustment, 'Enable', 'Off');
        set(handles.Sldr_Wavenumber_Adjustment, 'UserData', 2);
        % Set the flag for the next operation.
    case 2, % This is the translational calibration (step 1/2)
        UIControls_Enable(handles, 'Off');
        
        set(handles.STxt_Message, 'String', Messages(2));
        Index1_1 = Pick(Wavenumbers_R, Absorbance_R);
        Wavenumber_Translation_R = Wavenumbers_R(Index1_1);
        plot(Wavenumbers_R(Index1_1), Absorbance_R(Index1_1), 'g*');
        % The band selected on the reference spectrum for the calibration
        % of translation.
        set(handles.STxt_Message, 'String', Messages(3));
        Index2_1 = Pick(Wavenumbers_C, Absorbance_C);
        Wavenumber_Translation_C = Wavenumbers_C(Index2_1);
        set(gcf, 'UserData', Index2_1);
        % The band selected on the calibrated spectrum for the calibration
        % of translation.  The index (position of) to the peak of the band 
        % is a very important parameter in the whole calibration, and saved.
        
        set(handles.STxt_Message, 'String', Messages(4));
        Offset_Translation = Wavenumber_Translation_R - Wavenumber_Translation_C;
        if Offset_Translation == 0
            Offset_Translation = 0.5;
        end
        % If the two peaks show no wavenumber shift, a preset value for the
        % shift is given for manual adjustment.
        % The fifth modificaion.
        
        delete(Hndl_Plot_C);
        Hndl_Plot_C = plot(Wavenumbers_C + Offset_Translation, Absorbance_C);
        set(Hndl_Plot_C, 'Tag', 'Plot_Calibrated');
        % Calibration of translation was accomplished and displayed.
        
        UIControls_Enable(handles, 'On');
        set(handles.Sldr_Wavenumber_Adjustment, 'Value', 0);
        set(handles.Sldr_Wavenumber_Adjustment, 'Max', abs(Offset_Translation));
        set(handles.Sldr_Wavenumber_Adjustment, 'Min', -abs(Offset_Translation));
        % Make preparation for the fine adjustment.
        
        set(handles.Sldr_Wavenumber_Adjustment, 'UserData', 3);
        % Set the flag for the next operation.
    case 3, % This is the connection step
        set(handles.STxt_Message, 'String', Messages(5));
        
        OriginalCoordinateRanges = get(gca, 'UserData');
        set(gca, 'XLim', OriginalCoordinateRanges(1, :));
        set(gca, 'YLim', OriginalCoordinateRanges(2, :));
        % Restore the original display for the next step.
        
        set(handles.Sldr_Wavenumber_Adjustment, 'Enable', 'Off');
        set(handles.Sldr_Wavenumber_Adjustment, 'UserData', 4);
        % Set the flag for the next operation.
    case 4, % This is the dilative calibration (step 2/2)
        UIControls_Enable(handles, 'Off');
        
        set(handles.STxt_Message, 'String', Messages(6));
        Index1_2 = Pick(Wavenumbers_R, Absorbance_R);
        Wavenumber_Dilation_R = Wavenumbers_R(Index1_2);
        plot(Wavenumbers_R(Index1_2), Absorbance_R(Index1_2), 'g*');
        % The band selected on the reference spectrum for the calibration
        % of dilation.
        set(handles.STxt_Message, 'String', Messages(7));
        Index2_2 = Pick(Wavenumbers_C, Absorbance_C);
        Wavenumber_Dilation_C = Wavenumbers_C(Index2_2);
        Index_Translation = get(gcf, 'UserData');
        Index_Dilation = Index2_2;
        set(gcf, 'UserData', [Index_Translation; Index_Dilation]);
        % The band selected on the calibrated spectrum for the calibration
        % of dilation.  The index (position of) to the peak of the band is
        % also a very important parameter in the whole calibration, and saved.

        set(handles.STxt_Message, 'String', Messages(8));
        Offset_Dilation = Wavenumber_Dilation_R - Wavenumber_Dilation_C;
        if Offset_Dilation == 0
            Offset_Dilation = 0.5;
        end
        % If the two peaks show no wavenumber shift, a preset value for the
        % shift is given for manual adjustment.
        % The fifth modification.
        
        CalibratedWavenumber_Dilation = Wavenumber_Dilation_R;
        % The wavenumber which is calibrated for the dilation process.
        CalibratedWavenumber_Translation = Wavenumbers_C(Index_Translation);
        % The wavenumber which was already calibrated in the previous step.
        % It should be noted that the positions of the data points remain
        % unchanged during the whole process.
        CalibratedWavenumberIncrement = (CalibratedWavenumber_Dilation - ...
            CalibratedWavenumber_Translation)/...
            (Index_Dilation - Index_Translation);
        % The wavenumber increment after both translation and dilation
        % calibrations.
        % <<<< Calibration of dilation starts.
        DataLength = length(Wavenumbers_C);
        CalibratedWavenumbers = zeros(DataLength, 1);
        % Memory pre-allocation.
        CalibratedWavenumbers(Index_Translation-1:-1:1) = CalibratedWavenumber_Translation - ...
            (1:Index_Translation-1)'*CalibratedWavenumberIncrement;
        CalibratedWavenumbers(Index_Translation) = CalibratedWavenumber_Translation;
        CalibratedWavenumbers(Index_Translation+1:DataLength) = CalibratedWavenumber_Translation + ...
            (1:DataLength - Index_Translation)'*CalibratedWavenumberIncrement;
        % >>>> Calibration of dilation ends.
        delete(Hndl_Plot_C);
        Hndl_Plot_C = plot(CalibratedWavenumbers, Absorbance_C);
        set(Hndl_Plot_C, 'Tag', 'Plot_Calibrated');
        % Calibration of translation was accomplished and displayed.
        
        UIControls_Enable(handles, 'On');
        set(handles.Sldr_Wavenumber_Adjustment, 'Value', 0);
        set(handles.Sldr_Wavenumber_Adjustment, 'Max', abs(Offset_Dilation));
        set(handles.Sldr_Wavenumber_Adjustment, 'Min', -abs(Offset_Dilation));
        % Make preparation for the fine adjustment.
        
        set(handles.Sldr_Wavenumber_Adjustment, 'UserData', 5);
        % Set the flag for the next operation.
    case 5, %Generate the wavenumber-corrected spectrum by using linear interpolation.
        UIControls_Enable(handles, 'Off');
        set(hObject, 'Enable', 'Off');
        % Disable all the UI controls.  It is the time for the final step.

        OriginalCoordinateRanges = get(gca, 'UserData');
        set(gca, 'XLim', OriginalCoordinateRanges(1, :));
        set(gca, 'YLim', OriginalCoordinateRanges(2, :));
        % Restore the original display for the next step.
        
        tic;
        CalibratedWavenumbers = Wavenumbers_C;
        CalibratedAbsobance = Absorbance_C;
        CalibratedWavenumberIncrement = abs(CalibratedWavenumbers(1) - ...
            CalibratedWavenumbers(2));
        % Note the abscissa values of the curve in blue color were already
        % calibrated.
        Wavenumbers_Final = Wavenumbers_R;
        Absorbance_Final = zeros(length(Wavenumbers_Final), 1);
        for i = 1: length(Wavenumbers_Final),
            if Wavenumbers_Final(i) <= CalibratedWavenumbers(1)
                Absorbance_Final(i) = 0; % no information available.
            elseif Wavenumbers_R(i) >= CalibratedWavenumbers(end)
                NewAbsorbance_V(i) = 0;  % no information available either.
            else
                LeftBorder = max(find(Wavenumbers_R(i) > CalibratedWavenumbers));
                Slope = (Absorbance_C(LeftBorder + 1) - Absorbance_C(LeftBorder))/...
                    CalibratedWavenumberIncrement;
                Absorbance_Final(i) = Slope*(Wavenumbers_Final(i) - CalibratedWavenumbers(LeftBorder)) + ...
                    Absorbance_C(LeftBorder);
            end;
            ElapsingTime = toc;
            TimeTag1 = [num2str(fix(ElapsingTime/60)) ''' ' num2str(round(mod(ElapsingTime, 60))) ''''''];
            RemainingTime = (length(Wavenumbers_Final)/i - 1)*ElapsingTime;
            TimeTag2 = [num2str(fix(RemainingTime/60)) ''' ' num2str(round(mod(RemainingTime, 60))) ''''''];
            MSG = {'The final wavenumber-shift-calibrated spectrum is being generated.'; ...
                ['Calculations finished: ' num2str(i) ' / ' num2str(length(Wavenumbers_Final)) ...
                '.  Time elapsed: ' TimeTag1 '.  Time remain: ' TimeTag2]};
            set(handles.STxt_Message, 'String', MSG);
            drawnow;
        end;
        global ISASCENDINGORDER;
        if ~ISASCENDINGORDER
            Wavenumbers_Final = Wavenumbers_Final(end:-1:1);
            Absorbance_Final = Absorbance_Final(end:-1:1);
        end;
        % If necessary resort the final spectrum as the order of original
        % wavenumber to be calibrated.
        set(hObject, 'UserData', [Wavenumbers_Final(:) Absorbance_Final]);
        uiresume;
end;
% ____ End of function 'Btn_Proceed_Callback'.


% Function used to pick the peak in a region defined by two clicks.
function WavenumberIndex_out = Pick(Wavenumbers, Absorbance);
Region = ginput(2);
LeftBorder = min(Region(1, 1), Region(2, 1));
RightBorder = max(Region(1, 1), Region(2, 1));
% Borders of the region just determined by two clicks.

[UselessValue, LeftBorderIndex] = min(abs(Wavenumbers - LeftBorder));
[UselessValue, RightBorderIndex] = min(abs(Wavenumbers - RightBorder));
[Useless, Index] = max(Absorbance(LeftBorderIndex:RightBorderIndex));
WavenumberIndex_out = LeftBorderIndex + Index - 1;
% ____ End of function 'Pick'.


function ZoomInCommand(hObject, eventdata, handles)
Point_BtnDown = get(gca,'CurrentPoint');
finalRect = rbbox;
Point_BtnUp = get(gca,'CurrentPoint');
% Detect the Rubberband box.

Point_LB = min(Point_BtnDown(1,1:2), Point_BtnUp(1,1:2));
% The point at the left-bottom corner of the rectangle.
Offset = abs(Point_BtnDown(1,1:2) - Point_BtnUp(1,1:2));
% Dimensions of the rectangle.

XRange = get(gca, 'XLim'); YRange = get(gca, 'YLim');
J1 = (Point_LB(1) - XRange(1))*(XRange(2) - Point_LB(1)) > 0;
J2 = (Point_LB(1) + Offset(1) - XRange(1))*(XRange(2) - Point_LB(1) - Offset(1)) > 0;
J3 = (Point_LB(2) - YRange(1))*(YRange(2) - Point_LB(2)) > 0;
J4 = (Point_LB(2) + Offset(2) - YRange(1))*(YRange(2) - Point_LB(2) - Offset(2)) > 0;
J5 = Offset > 0;
if J1*J2*J3*J4*J5,
    set(gca, 'XLim', [Point_LB(1) Point_LB(1) + Offset(1)]);
    set(gca, 'YLim', [Point_LB(2) Point_LB(2) + Offset(2)]);
end;
% Make sure the rubberband box is inside the axes.

Hndl_TglBtn_ZoomIn = findobj('Tag', 'TglBtn_ZoomIn');
set(Hndl_TglBtn_ZoomIn, 'Value', get(Hndl_TglBtn_ZoomIn, 'Min'));
set(gca, 'ButtonDownFcn', ' ');
% Make the 'Zoom In' button up: this zoom-in operation is done.
set(gcf, 'Pointer', 'Arrow');
% Restore the normal mouse pointer.
% ____ End of function 'ZoomInCommand'.


% Function used to set the 'Enable' status of 4 UI controls.
function UIControls_Enable(handles, SwitchStatus)
set(handles.TglBtn_ZoomIn, 'Enable', SwitchStatus);
set(handles.Btn_Restore, 'Enable', SwitchStatus);
set(handles.Sldr_Wavenumber_Adjustment, 'Enable', SwitchStatus);
set(handles.Btn_Proceed, 'Enable', SwitchStatus);
% ____ End of function 'UIControls_Enable'.


% Function used to generate different messages.
function MessageNeeded = Messages(MessageIndex)
Msg{1} = {['There are 2 steps in the wavenumber shift calibration.  ' ...
    'Step #1 needs be performed in the LOW-WAVENUMBER region.  ' ...
    'After necessary zooming in, press ''Proceed'' to continue.']};
Msg{2} = {['Step #1 (LOW-WAVENUMBER): Now on the reference spectrum ' ...
    'select a region in which a well-separated band has largest ' ...
    'absorbance.  Select in this way: the first and the second ' ...
    'clicks determine the left and the right borders.']};
Msg{3} = {['Step #1 (Low_WAVENUMVER): Now do the same thing to the ' ...
    'calibrated spectrum.  Make sure the counterpart of that band is ' ...
    'in the region.']};
Msg{4} = {['Step #1 is done!  However, you can use the slider to make ' ...
    'some fine adjustments.  When the result is satisfactory, press ' ...
    '''Proceed'' to continue.']};
Msg{5} = {['It is the time for the next step.  Step #2 needs be ' ...
    'performed in the HIGH-WAVENUMBER region.  After necessary ' ...
    'zooming in, press ''Proceed'' to continue.']};
Msg{6} = {['Step #2 (HIGH-WAVENUMBER): Now on the reference spectrum ' ...
    'select a region in which a well-separated band has largest ' ...
    'absorbance.  Select in this way: the first and the second ' ...
    'clicks determine the left and the right borders.']};
Msg{7} = {['Step #2 (HIGH_WAVENUMVER): Now do the same thing to the ' ...
    'calibrated spectrum.  Make sure the counterpart of that band is ' ...
    'in the region.']};
Msg{8} = {['Step #2 is done!  However, you can use the slider to make ' ...
    'some fine adjustments.  When the result is satisfactory, press ' ...
    '''Proceed'' to continue.']};

MessageNeeded = Msg{MessageIndex};
% ____ End of function 'Messages'.


% --- Executes when user attempts to close Main_Window.
function Main_Window_CloseRequestFcn(hObject, eventdata, handles)
% hObject    handle to Main_Window (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

if get(handles.Sldr_Wavenumber_Adjustment, 'UserData') ~= 5
    set(handles.Btn_Proceed, 'UserData', [0 0]);
    disp(' ');
    disp('Error!');
    disp('The program was abnormally aborted.');
    disp('The output is meaningless.');
end;
uiresume;

